###########################################################################################
# Replication do-file for the random forest classification, this file is not required to replicate the tables, and is only supplied for documentation purposes
# Paper:                  He, Schndeln, Jaimovich: Development Projects and Economic Networks: Lessons From Rural Gambia
# Journal:                Review of Economic Studies
# Contact:                Simon He (hess@econ.uni-frankfurt.de)
###########################################################################################



setwd("D:/Replication files/data")

#Clear Workspace
rm(list = ls())
options(warn=-1)

#Load package dependencies
library(readstata13)  #  0.9.2
library(foreign)      #  0.8.75
library(randomForest) #  4.6.14
library(dplyr)        #  0.8.5

#Load household-level data
dat0<-read.dta13("households.dta")

#use inverse path lengths (functional forms don't mater anyways, and this is an easy way to impute 0/infty for dyads which are not connected)
dat0$alkalo_path_length_fam<-1/(dat0$alkalo_path_length_fam+1)
dat0$alkalo_path_length_fam[which(is.na(dat0$alkalo_path_length_fam))]<-0

dat0$alkalo_path_length<-1/(dat0$alkalo_path_length+1)
dat0$alkalo_path_length[which(is.na(dat0$alkalo_path_length))]<-0

#g16 comes straight from the survey (one household reported NA). Also the two shortest time frames (<1year and 1-5 years) have very few obsrevations, so we bundle them
dat0$g16[which(dat0$g16=="Don't know/no answer")]<-NA
dat0$g16[which(dat0$g16=="Recently (< 1 year)")]<-"Some years (between 1 and 5 years)"

#combine into a rough numeric variable (again, the functional form should not matter for RF) to make use of the ordinal nature of that variable
dat0$duration<- 1 * (dat0$g16== "Some years (between 1 and 5 years)") + 
                2 * (dat0$g16== "Many years (> 5 years, but not born in the village)") +
                3 * (dat0$g16== "Parents migrate to the village") +
                3 * (dat0$g16== "Family in the village for many generations (parents were born in the village)") 

#combine some small factor levels and fix coding
dat0$ethnicity[which(dat0$ethnicity==7)]<-8
dat0$ethnicity[which(dat0$ethnicity==9)]<-8
dat0$ethnicity<-as.factor(dat0$ethnicity)
dat0$ward_2003[which(dat0$ward_2003==8)]<-7 #8 is very small not enough treatment villages, but right next to as 7, so we merge them into 1
dat0$ward_2003<-as.factor(dat0$ward_2003)
dat0$lga<-as.factor(dat0$lga)

#replace missings with village-means so that they are not dropped
dat0<-dat0 %>% 
  group_by(village) %>% 
  mutate(
    vmeanshortestpath=mean(meanshortestpath,na.rm = TRUE),
    vmeangeodist=mean(meangeodist,na.rm = TRUE),
    vmeaninvshortpath_fam=mean(meaninvshortpath_fam,na.rm = TRUE)
  )
dat0$meaninvshortpath_fam[which(is.na(dat0$meaninvshortpath_fam))]<-dat0$vmeaninvshortpath_fam[which(is.na(dat0$meaninvshortpath_fam))]
dat0$meangeodist[which(is.na(dat0$meangeodist))]<-dat0$vmeangeodist[which(is.na(dat0$meangeodist))]
dat0$meanshortestpath[which(is.na(dat0$meanshortestpath))]<-dat0$vmeanshortestpath[which(is.na(dat0$meanshortestpath))]

#Set of variables to be used for the classification
features_survey<-c("alkalo","imam","alkalo_relative","age","femalehead","alkalo_path_length","alkalo_path_length_fam","meanshortestpath","meaninvshortpath_fam","meangeodist","ethnicity_vill_share","ethn_majority","intdeg_undir_m8a8b8c","extdeg_undir_m8a8b8c","formal_educ","coranic_educ", "hhsize", "ha_land","compound_head","wives","g16","ward_2003","ethnicity")

#A (sub)set of varaibles that are also available from the census
features_census<-c("age","femalehead", "ethnicity_vill_share","ethn_majority","ethnicity","formal_educ","hhsize", "wives","duration","lga")

#only treatment villages have the variable we care about (elite_new)  
datT<-subset(dat0,treatment=="Treatment", select = c("uid","elite_new","village",union(features_survey,features_census)))
  
#fix NAS
set.seed(2019)
datT<-na.roughfix(datT)

#prepare empty list for reslt
leave_one_out_prediction<-NULL

#loop over T-villages, train a model without the current village and obtain predictions
for(vill in unique(datT$village)){

    #training data
    datT$train=datT$village!=vill 
    TrainSet <- datT[datT$train,]
    
    dat2use <- subset(TrainSet,!is.na(TrainSet$elite_new),select=c("elite_new",union(features_survey,features_census)))
    
    #train one model based on all T-villages but the current one
    set.seed(2019)
    model_survey <- randomForest(as.factor(elite_new) ~ ., dat = subset(dat2use,subset=T,select=c("elite_new",features_survey)), importance = TRUE,na.action=na.roughfix, ntree=1000)

    #predict
    datT$predValid_l1_survey <- predict(model_survey, datT,type="prob")[,2]

    #append to output, but only for the one that wasn't  used for training
    leave_one_out_prediction<-rbind(leave_one_out_prediction, subset(datT,train==FALSE, select=c("uid","elite_new",grep("pred",names(datT),value=TRUE))))
    
}
#name columns
names(leave_one_out_prediction)<-c("uid",paste0("elite_new",c("_out_l1o",grep("pred",names(datT),value=TRUE))))
  
#train on all T villages, to predict the control villages
set.seed(2019)
model_full_training_survey <- randomForest(as.factor(elite_new) ~ ., dat = subset(datT,subset=!is.na(elite_new),select=c("elite_new",features_survey), importance = TRUE),na.action = na.roughfix, ntree=1000)

#train on all T villages, to predict the census villages, this will be used later
set.seed(2019)
model_full_training_census <- randomForest(as.factor(elite_new) ~ ., dat = subset(datT,subset=!is.na(elite_new),select=c("elite_new",features_census), importance = TRUE),na.action = na.roughfix, ntree=1000)
  

#predict for all control villages
dat0$predValid_s_f <- predict(model_full_training_survey, newdata=na.roughfix(dat0[,features_survey]),type="prob")[,2]

#merge, usins the leave-one-out prediction for treatment villages and the full ones for control villages
result<-merge(dat0,leave_one_out_prediction,by=c("uid"),all=TRUE)
result$elite_new_RF<-ifelse(result$treatment!="Treatment",result$predValid_s_f,result$elite_newpredValid_l1_survey)
write.dta(result[,c("uid","elite_new_RF")],"replication_RF.dta")

  
#Now we use the second model to predict VDC for the census data:
census_data <-read.dta13("census.dta")
#make sure all varaibles are in the right format
census_data$age <- as.integer(census_data$RF_age)
census_data$femalehead <- as.integer(census_data$RF_femalehead)
census_data$ethn_majority <- as.integer(census_data$RF_ethn_majority)
census_data$formal_educ <- as.integer(census_data$RF_formal_educ)
census_data$coranic_educ <- as.integer(census_data$RF_coranic_educ)
census_data$hhsize <- as.integer(census_data$RF_hhsize)
census_data$wives <- as.integer(census_data$RF_wives)
census_data$ethnicity_vill_share <- as.numeric(census_data$RF_ethnicity_vill_share)
census_data$ethnicity <- as.factor(census_data$RF_ethnicity)
census_data$lga <- as.factor(census_data$RF_lga)
census_data$duration <- as.numeric(census_data$RF_duration)

#predict
census_data$RF_predicted_elite_new <- predict(model_full_training_census, newdata=na.roughfix(census_data[,features_census]),type="prob")[,2]
#save
census_data <- write.dta(census_data,"replication_census4RF.dta")

  